/*
 * SampleSales 
 *
 * 04/09/2004
 *
 * Copyright:
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * xxxx-xxx
 * (C) Copyright IBM Corp. 2004.
 *
 */
package sample;

import java.rmi.RemoteException;
import java.util.Iterator;

import com.ibm.retail.saf.provider.kiosk.KioskPOSProvider;
import com.ibm.retail.saf.provider.kiosk.KioskPOSProviderFactory;
import com.ibm.retail.saf.provider.kiosk.KioskRequestFactory;
import com.ibm.retail.saf.provider.ApplicationException;
import com.ibm.retail.saf.provider.CashReceiptImage;
import com.ibm.retail.saf.provider.CreditPaymentInfo;
import com.ibm.retail.saf.provider.ItemIdentifier;
import com.ibm.retail.saf.provider.POSException;
import com.ibm.retail.saf.provider.POSRequestFactory;
import com.ibm.retail.saf.provider.PaymentRequest;
import com.ibm.retail.saf.provider.PaymentResult;
import com.ibm.retail.saf.provider.PriceInfo;
import com.ibm.retail.saf.provider.PriceRequest;
import com.ibm.retail.saf.provider.PriceResult;
import com.ibm.retail.saf.provider.PrintLine;
import com.ibm.retail.saf.provider.ProviderRequest;
import com.ibm.retail.saf.provider.ReceiptLineItem;
import com.ibm.retail.saf.provider.RecoveryException;
import com.ibm.retail.saf.provider.RequestException;
import com.ibm.retail.saf.provider.SalesTransaction;
import com.ibm.retail.saf.provider.TimeoutException;
import com.ibm.retail.saf.provider.TransactionInProgressException;
import com.ibm.retail.saf.provider.TransactionRequest;
import com.ibm.retail.saf.provider.TransactionResult;

/**
 * Provides sample source code demonstrating the POSProvider API.
 * 
 */
public class SampleSales
{
    // Terminal number will be supplied by the kiosk configuration
    public static String              POS_TERMINAL_NUMBER = "200";

    // Instance data   
    protected KioskPOSProvider        provider            = null;
    protected KioskPOSProviderFactory providerFactory     = null;
    protected POSRequestFactory       requestFactory      = null;

    /**
     * 
     * Default constructor.
     * 
     */
    public SampleSales()
    {
        // Get singleton instances of the client side factories
        providerFactory = (KioskPOSProviderFactory) KioskPOSProviderFactory.getInstance();
        requestFactory = KioskRequestFactory.getInstance();
    }

    /**
     * Create an instance of a Kiosk POSProvider. 
     * 
     * @return true if an instance of POSProvider is created successfully
     */
    public boolean createPOSProvider()
    {
        boolean result = false;

        // Make a request for a POS provider instance for this client      
        // and create the POSProvider instance.      
        ProviderRequest providerRequest = requestFactory.makeProviderRequest();
        providerRequest.setPOSTerminalNumber(POS_TERMINAL_NUMBER);
        try
        {
            System.out.println("\n[Creating POSProvider...]");
            System.out.println("-------------------------");
            provider = (KioskPOSProvider) providerFactory.makePOSProvider(providerRequest);
            result = true;
            System.out.println("Created successfully");

        }
        catch (TimeoutException e)
        {
            System.err.println("Timed out creating the POSProvider");
            System.err.println("Message is " + e.getMessage());
        }
        catch (POSException e)
        {
            System.err.println("An error occurred creating the POSProvider");
            System.err.println("Message is " + e.getMessage());
        }
        catch (RemoteException e)
        {
            System.err.println("An error occurred connecting to the remote POS");
            System.err.println("Message is " + e.getMessage());
        }

        return result;
    }

    /**
     * Test to seee if POS is in position to be used
     * 
     */
    public boolean testPOSActive()
    {
        boolean active = false;
        long time1 = 0, time2 = 0;

        // Check to determine if the POS is active
        try
        {
            System.out.println("\n[Checking if POS is OK ...]");
            System.out.println("---------------------------");
            active = provider.isPOSActive();
            System.out.println("POS is OK");
        }
        catch (POSException e)
        {
            System.err.println("POS is unavailable");
            System.err.println("Message is " + e.getMessage());
            if (e.getCause() != null)
            {
                System.err.println("Cause is " + e.getCause());
            }
        }
        catch (RecoveryException e)
        {
            System.err.println("Wait - POS in recovery mode");
            System.err.println("Message is " + e.getMessage());
            if (e.getCause() != null)
            {
                System.err.println("Cause is " + e.getCause());
            }
        }
        catch (RemoteException e)
        {
            System.err.println("Connection Error");
            System.err.println("Message is " + e.getMessage());
            if (e.getCause() != null)
            {
                System.err.println("Cause is " + e.getCause());
            }
        }
        return active;
    }

    /**
     * Use the POSProvider to get base item prices from the point-of-sale system.
     * 
     */
    public void performPriceCheck()
    {
        long time1 = 0, time2 = 0;

        // Create a price request and send it to the POS system.

        try
        {
            System.out.println("\n[Performing a Base Item price check]...");
            System.out.println("---------------------------------------");

            // Create an empty price request
            PriceRequest priceRequest = requestFactory.makePriceRequest();

            // Add the item identifiers
            // A default item identifier is treated as an item barcode by the POS
            priceRequest.addItem(requestFactory.makeDefaultItemIdentifier("1"));
            priceRequest.addItem(requestFactory.makeDefaultItemIdentifier("2"));
            priceRequest.addItem(requestFactory.makeDefaultItemIdentifier("3"));
            priceRequest.addItem(requestFactory.makeDefaultItemIdentifier("4"));
            priceRequest.addItem(requestFactory.makeDefaultItemIdentifier("5"));
            priceRequest.addItem(requestFactory.makeDefaultItemIdentifier("6"));
            priceRequest.addItem(requestFactory.makeDefaultItemIdentifier("7"));
            priceRequest.addItem(requestFactory.makeDefaultItemIdentifier("8"));
            priceRequest.addItem(requestFactory.makeDefaultItemIdentifier("9"));
            priceRequest.addItem(requestFactory.makeDefaultItemIdentifier("10"));

            // Execute the request on the point-of-sale system
            // Call is synchronous and client blocks until result is received
            time1 = System.currentTimeMillis();
            PriceResult priceResult = provider.getBaseItemPrices(priceRequest);
            time2 = System.currentTimeMillis();
            System.out.println("Total time (ms) = " + (time2 - time1));

            // Get the price list iterator from the result and output the results
            System.out.println("\n[Getting LineItems from PriceResult]...");
            System.out.println("---------------------------------------");

            Iterator items = priceResult.itemIdentifiers();
            System.out.println("Total # of Items in result = " + priceResult.getNumberOfPrices());
            while (items.hasNext())
            {
                ItemIdentifier itemID = (ItemIdentifier) (items.next());
                System.out.println("Get price for Item = " + itemID.toString());
                try
                {
                    PriceInfo priceInfo = priceResult.getPrice(itemID);
                    if (priceInfo != null)
                    {
                        System.out.println("Item: " + itemID.toString() + " Price: " + priceInfo.getRegularPrice());
                    }
                    else
                    {
                        System.out.println("Item: " + itemID.toString() + " *** No Price Found ***");
                    }
                }
                catch (POSException pe)
                {
                    System.out.println("Item: " + itemID.toString() + " *** No Price Found ***");
                }
            }
        }
        // Handle POSExceptions thrown by the POSProvider
        // Note: This could also be implemented as a single catch block for POSException
        catch (TransactionInProgressException t)
        {
            // Indicates that the POS was still processing a transaction
            System.err.println("Logic Error:  A transaction is in progress.");
        }
        catch (RequestException t)
        {
            // Indicates that the request contained invalid or missing data
            System.err.println("Logic Error:  Price request contains invalid or missing data.");
        }
        catch (RemoteException e)
        {
            // Indicates a problem with the connection to the remote POS
            System.err.println("Problem connecting to the remote POS:  Price checker temporarily unavailable.");
            System.err.println("  Message is " + e.getMessage());
            if (e.getCause() != null)
            {
                System.err.println("  Cause is " + e.getCause());
            }
        }
        catch (TimeoutException e)
        {
            // Timed out trying to get prices 
            //  -- Could indicate timeout value is too low or POS is not available
            time2 = System.currentTimeMillis();
            System.err.println("Request timed out:  Price checker temporarily unavailable.");
            System.out.println("Total time (ms) = " + (time2 - time1));
            System.err.println("  Message is " + e.getMessage());
        }
        catch (ApplicationException e)
        {
            // Indicates a problem processing the request at the POS system
            // -- Typically indicates a bad configuration or application error on the POS 
            System.err.println("Problem processing request at POS:  Price checker is out of service.");
            System.err.println("  Message is " + e.getMessage());
            if (e.getCause() != null)
            {
                System.err.println("  Cause is " + e.getCause());
            }
        }
        catch (POSException e)
        {
            // Handle all other POSExceptions 
            System.err.println("Problem accessing POS:  Price check failed");
            System.err.println("  Message is " + e.getMessage());
            if (e.getCause() != null)
            {
                System.err.println("  Cause is " + e.getCause());
            }
        }
        catch (RecoveryException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * Use the POSProvider to perform sales and pay balance using MSR data
     * 
     */
    public void performSalesAndPayBalanceUsingMSR()
    {
        long time1 = 0, time2 = 0;

        // Create a price request and send it to the POS system.
        try
        {
            System.out.println("\n[Performing sales order...]");
            System.out.println("---------------------------");

            SalesTransaction transaction = provider.makeSalesTransaction();

            // Create an empty price request
            TransactionRequest transactionRequest = requestFactory.makeTransactionRequest();

            // Add the item identifiers
            // A default item identifier is treated as an item barcode by the POS
            transactionRequest.addLineItem(requestFactory.makeDefaultLineItem("04177189115", "5"));
            transactionRequest.addLineItem(requestFactory.makeDefaultLineItem("04177847410", "2"));
            transactionRequest.addLineItem(requestFactory.makeDefaultLineItem("04177136545", "1"));

            // Execute the request on the point-of-sale system
            // Call is synchronous and client blocks until result is received
            time1 = System.currentTimeMillis();
            TransactionResult transactionResult = transaction.performSales(transactionRequest);
            time2 = System.currentTimeMillis();
            System.out.println("Time taken to price this sales order(ms) = " + (time2 - time1));

            System.out.println("\nGetting Receipt Line Items from Transaction Result...");
            // Get the price list iterator from the result and output the results
            Iterator items = transactionResult.getReceiptLineItems();
            while (items.hasNext())
            {
                ReceiptLineItem lineItem = (ReceiptLineItem) (items.next());
                System.out.println("Item: " + lineItem.getDescription() + " Price: " + lineItem.getExtendedPrice());
            }
            System.out.println("\n[Performing payment process with MSR data...]");

            // Create an empty payment request and an empty credit payment info
            PaymentRequest paymentRequest = requestFactory.makePaymentRequest();
            CreditPaymentInfo creditCardInfo = requestFactory.makeCreditPaymentInfo();

            // Add the payment details to the credit card info
            byte[] track1Data = {34, 20, 18, 22, 20, 18, 25, 24, 19, 24, 23, 22, 16, 17, 16, 20, 19, 62, 55, 40, 41, 52, 44, 37, 57, 15, 50, 0, 39, 33, 41, 46,
                    37, 51, 62, 16, 21, 16, 21, 17, 16, 17, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
                    24, 25, 17, 16, 16, 16, 16, 16, 16};
            byte[] track2Data = {4, 4, 4, 5, 2, 2, 2, 2, 9, 9, 9, 9, 0, 0, 0, 7, 13, 10, 0, 0, 4, 1, 0, 1, 0, 0, 0, 0, 0, 8, 9, 1, 0, 0, 0, 0, 0};
            creditCardInfo.setCardType("visa");
            creditCardInfo.setTrackData(1, track1Data);
            creditCardInfo.setTrackData(2, track2Data);
            paymentRequest.setPaymentInformation(creditCardInfo);

            // Execute the request on the point-of-sale system
            // Call is synchronous and client blocks until result is received
            time1 = System.currentTimeMillis();
            PaymentResult paymentResult = transaction.payBalance(paymentRequest);
            echoPaymentResult(paymentResult);
            time2 = System.currentTimeMillis();
            System.out.println("Time taken to perform credit transaction(ms) = " + (time2 - time1));
            // transaction.cancel();

            System.out.println("\nViewing CashReceiptImage print lines...");
            CashReceiptImage cashReceipt = transaction.getCashReceiptImage();
            String aReceiptLine;
            Iterator it1 = cashReceipt.getPrintLines();
            //                 "123456789012345678901234567890123456789"
            System.out.println("\n[------- cash receipt start -----------]");
            while (it1.hasNext())
            {
                aReceiptLine = ((PrintLine) it1.next()).getText();
                System.out.println(aReceiptLine);
            }
            System.out.println("[------- cash receipt end   -----------]");

        }
        // Handle POSExceptions thrown by the POSProvider
        // Note: This could also be implemented as a single catch block for POSException
        catch (TransactionInProgressException t)
        {
            // Indicates that the POS was still processing a transaction
            System.err.println("Logic Error:  A transaction is in progress. Specific Exception:" + t);
        }
        catch (RequestException t)
        {
            // Indicates that the request contained invalid or missing data
            System.err.println("Logic Error:  Request contains invalid or missing data." + t);
        }
        catch (RemoteException e)
        {
            // Indicates a problem with the connection to the remote POS
            System.err.println("Problem connecting to the remote POS:  Sales Processor temporarily unavailable.");
            System.err.println("  Message is " + e.getMessage());
            if (e.getCause() != null)
            {
                System.err.println("  Cause is " + e.getCause());
            }
        }
        catch (TimeoutException e)
        {
            // Timed out trying to get prices 
            //  -- Could indicate timeout value is too low or POS is not available
            time2 = System.currentTimeMillis();
            System.err.println("Request timed out:  Sales Processor temporarily unavailable.");
            System.out.println("Total time (ms) = " + (time2 - time1));
            System.err.println("  Message is " + e.getMessage());
        }
        catch (ApplicationException e)
        {
            // Indicates a problem processing the request at the POS system
            // -- Typically indicates a bad configuration or application error on the POS 
            System.err.println("Problem processing request at POS:  Sales Processor is out of service.");
            System.err.println("  Message is " + e.getMessage());
            if (e.getCause() != null)
            {
                System.err.println("  Cause is " + e.getCause());
            }
        }
        catch (POSException e)
        {
            // Handle all other POSExceptions 
            System.err.println("Problem accessing POS:  Sales Processing failed");
            System.err.println("  Message is " + e.getMessage());
            if (e.getCause() != null)
            {
                System.err.println("  Cause is " + e.getCause());
            }
        }
        catch (RecoveryException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * Use the POSProvider to perform sales and pay balance using MSR data
     * 
     */
    public void performSalesAndPayBalanceUsingKeyEntry()
    {
        long time1 = 0, time2 = 0;

        // Create a price request and send it to the POS system.
        try
        {
            System.out.println("\n[Performing sales order...]");
            System.out.println("---------------------------");

            SalesTransaction transaction = provider.makeSalesTransaction();

            // Create an empty price request
            TransactionRequest transactionRequest = requestFactory.makeTransactionRequest();

            // Add the item identifiers
            // A default item identifier is treated as an item barcode by the POS
            transactionRequest.addLineItem(requestFactory.makeDefaultLineItem("04177189115", "5"));
            transactionRequest.addLineItem(requestFactory.makeDefaultLineItem("04177847410", "2"));
            transactionRequest.addLineItem(requestFactory.makeDefaultLineItem("04177136545", "1"));

            // Execute the request on the point-of-sale system
            // Call is synchronous and client blocks until result is received
            time1 = System.currentTimeMillis();
            TransactionResult transactionResult = transaction.performSales(transactionRequest);
            time2 = System.currentTimeMillis();

            System.out.println("\nGetting Receipt Line Items from Transaction Result...");

            // Get the price list iterator from the result and output the results
            Iterator items = transactionResult.getReceiptLineItems();
            while (items.hasNext())
            {
                ReceiptLineItem lineItem = (ReceiptLineItem) (items.next());
                System.out.println("Item: " + lineItem.getDescription() + " Price: " + lineItem.getExtendedPrice());
            }
            System.out.println("Time taken to price this sales order(ms) = " + (time2 - time1));

            System.out.println("\n[Perform payment process with keyed account entry...]");
            System.out.println("---------------------------------------------------");

            // Create an empty payment request and an empty credit payment info
            PaymentRequest paymentRequest = requestFactory.makePaymentRequest();
            CreditPaymentInfo creditCardInfo = requestFactory.makeCreditPaymentInfo();

            // Add the payment details to the credit card info
            String accountNumber = "4445222299990007";
            String date = "1004";
            creditCardInfo.setCardType("visa");
            creditCardInfo.setAccountNumber(accountNumber);
            creditCardInfo.setExpirationDate(date);
            paymentRequest.setPaymentInformation(creditCardInfo);

            // Execute the request on the point-of-sale system
            // Call is synchronous and client blocks until result is received
            time1 = System.currentTimeMillis();
            PaymentResult paymentResult = transaction.payBalance(paymentRequest);
            time2 = System.currentTimeMillis();
            System.out.println("\nGetting Payment Results...");
            System.out.println("--------------------------");

            echoPaymentResult(paymentResult);
            System.out.println("Time taken to perform credit transaction(ms) = " + (time2 - time1));

            // Print the cash receipt image
            System.out.println("\nViewing CashReceiptImage print lines...");
            System.out.println("---------------------------------------");

            CashReceiptImage cashReceipt = transaction.getCashReceiptImage();
            String aReceiptLine;
            Iterator it1 = cashReceipt.getPrintLines();
            //                 "  123456789012345678901234567890123456789  "
            System.out.println("\n[ ------- CASH RECEIPT START -----------  ]\n");
            while (it1.hasNext())
            {
                aReceiptLine = ((PrintLine) it1.next()).getText();
                System.out.println("| " + aReceiptLine);
            }
            System.out.println("[ ------- CASH RECEIPT END  -----------  ]");

            System.out.println("\nConfirming payment acknowledged...");
            // If we got here, then we got a good result. Go ahead and confirm the transaction
            transaction.confirm();
            System.out.println("Confirmation received OK.");

        }
        // Handle POSExceptions thrown by the POSProvider
        // Note: This could also be implemented as a single catch block for POSException
        catch (TransactionInProgressException t)
        {
            // Indicates that the POS was still processing a transaction
            System.err.println("Logic Error:  A transaction is in progress. Specific Exception:" + t);
        }
        catch (RequestException t)
        {
            // Indicates that the request contained invalid or missing data
            System.err.println("Logic Error:  Request contains invalid or missing data." + t);
        }
        catch (RemoteException e)
        {
            // Indicates a problem with the connection to the remote POS
            System.err.println("Problem connecting to the remote POS: Sales Processor temporarily unavailable.");
            System.err.println("  Message is " + e.getMessage());
            if (e.getCause() != null)
            {
                System.err.println("  Cause is " + e.getCause());
            }
        }
        catch (TimeoutException e)
        {
            // Timed out trying to handle sales request 
            //  -- Could indicate timeout value is too low or POS is not available
            time2 = System.currentTimeMillis();
            System.err.println("Request timed out:  Sales Processor temporarily unavailable.");
            System.out.println("Total time (ms) = " + (time2 - time1));
            System.err.println("  Message is " + e.getMessage());
        }
        catch (ApplicationException e)
        {
            // Indicates a problem processing the request at the POS system
            // -- Typically indicates a bad configuration or application error on the POS 
            System.err.println("Problem processing request at POS:  Sales Processor is out of service.");
            System.err.println("  Message is " + e.getMessage());
            if (e.getCause() != null)
            {
                System.err.println("  Cause is " + e.getCause());
            }
        }
        catch (POSException e)
        {
            // Handle all other POSExceptions 
            System.err.println("Problem accessing POS:  Sales Processing failed");
            System.err.println("  Message is " + e.getMessage());
            if (e.getCause() != null)
            {
                System.err.println("  Cause is " + e.getCause());
            }
        }
        catch (RecoveryException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
    /**
     * Use the POSProvider to perform sales and pay balance using MSR data
     * 
     */
    public void echoPaymentResult(PaymentResult result)
    {
        System.out.println("Note: Values currently null ");
        System.out.println("ApprovalCode = " + result.getApprovalCode());
        System.out.println("MaskedAccountNumber = " + result.getMaskedAccountNumber());
        System.out.println("ReferenceNumber = " + result.getReferenceNumber());
        System.out.println("ResponseCode = " + result.getResponseCode());
        System.out.println("SequenceNumber = " + result.getSequenceNumber());
        Iterator legalTextIterator = result.getLegalText();
        int i = 1;
        while (legalTextIterator.hasNext())
        {
            System.out.println("Legal Text Line:" + i + " =" + legalTextIterator.next());
            i++;
        }
        System.out.println("\n");

    }

    /**
     * Releases all point-of-sale resources associated with this POSProvider.
     */
    public void release()
    {
        try
        {
            provider.release();
        }
        catch (POSException e)
        {
            System.err.println("Problem accessing POS:  release failed");
            System.err.println("  Message is " + e.getMessage());
            if (e.getCause() != null)
            {
                System.err.println("  Cause is " + e.getCause());
            }
        }
        catch (RemoteException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * Main routine.
     * 
     * <ol>
     * <li>Create a KioskPOSProvider</li>
     * <li>Perform a price check</li>
     * <li>Release resources</li>
     * <li>Exit</li>
     * </ol>
     * 
     */
    public static void main(String[] args)
    {
        String terminalNumber = System.getProperty("aefTerm");
        if (terminalNumber != null)
        {
            POS_TERMINAL_NUMBER = terminalNumber;
        }

        SampleSales client = new SampleSales();
        boolean providerAvailable = client.createPOSProvider();
        if (providerAvailable)
        {
            if (client.testPOSActive())
            {
                client.performPriceCheck();
                client.performSalesAndPayBalanceUsingKeyEntry();
                client.release();
            }
        }
    }
}